@@ -3,24 +3,31 @@
 from __future__ import annotations
 
 import os
+import re
 import sys
+from collections.abc import Callable
+from collections.abc import Hashable
+from collections.abc import Mapping
+from collections.abc import MutableMapping
 from functools import wraps
 from importlib import import_module
 from importlib.metadata import distribution
 from types import ModuleType
 from typing import TYPE_CHECKING
 from typing import Any
-from typing import Callable
 from typing import TypeVar
 from typing import cast
 from warnings import warn
 
+import numpy as np
+
 DecoratedCallable = TypeVar(
     "DecoratedCallable", bound=Callable[..., Any]
 )  # decorated function or class
 
 if TYPE_CHECKING:
-    from typing import Sequence
+    from collections.abc import Sequence
+    from typing import NoReturn
 
 
 class OverDueDeprecation(Exception):
@@ -32,6 +39,33 @@ class OverDueDeprecation(Exception):
     warn_deprecated
     deprecate_module_attribute
     deprecate_submodule
+    deprecate_dict_entry
+    """
+
+
+class GlotaranDeprecatedApiError(Exception):
+    """Exception raised when a deprecation has no replacement.
+
+    See Also
+    --------
+    deprecate
+    warn_deprecated
+    deprecate_module_attribute
+    deprecate_submodule
+    deprecate_dict_entry
+    """
+
+
+class GlotaranApiDeprecationWarning(UserWarning):
+    """Warning to give users about API changes.
+
+    See Also
+    --------
+    deprecate
+    warn_deprecated
+    deprecate_module_attribute
+    deprecate_submodule
+    deprecate_dict_entry
     """
 
 
@@ -72,16 +106,18 @@ def parse_version(version_str: str) -> tuple[int, int, int]:
         If ``version_str`` 's first three elements can not be casted to int.
     """
     error_message = (
-        "version_str need to be a fully qualified version consisting of "
+        "version_str needs to be a fully qualified version consisting of "
         f"int parts (e.g. '0.0.1'), got {version_str!r}"
     )
     split_version = version_str.partition("-")[0].split(".")
     if len(split_version) < 3:
         raise ValueError(error_message)
     try:
-        return tuple(map(int, split_version[:3]))  # type:ignore [return-value]
-    except ValueError:
-        raise ValueError(error_message)
+        return tuple(
+            map(int, (*split_version[:2], split_version[2].partition("rc")[0]))
+        )  # type:ignore[return-value]
+    except ValueError as err:
+        raise ValueError(error_message) from err
 
 
 def check_qualnames_in_tests(qual_names: Sequence[str], importable_indices: Sequence[int]):
@@ -120,6 +156,78 @@ def check_qualnames_in_tests(qual_names: Sequence[str], importable_indices: Sequ
                 hasattr(item, qual_name_parts[-slice_index + 1])
 
 
+def check_overdue(deprecated_qual_name_usage: str, to_be_removed_in_version: str) -> None:
+    """Check if a deprecation is overdue for removal.
+
+    Parameters
+    ----------
+    deprecated_qual_name_usage : str
+        Old usage with fully qualified name e.g.:
+        ``'glotaran.read_model_from_yaml(model_yml_str)'``
+    to_be_removed_in_version : str
+        Version the support for this usage will be removed.
+
+    Raises
+    ------
+    OverDueDeprecation
+        If the current version is greater or equal to ``to_be_removed_in_version``.
+    """
+    if (
+        parse_version(glotaran_version()) >= parse_version(to_be_removed_in_version)
+        and "dev" not in glotaran_version()
+    ):
+        raise OverDueDeprecation(
+            f"Support for {deprecated_qual_name_usage.partition('(')[0]!r} was "
+            f"supposed to be dropped in version: {to_be_removed_in_version!r}.\n"
+            f"Current version is: {glotaran_version()!r}"
+        )
+
+
+def raise_deprecation_error(
+    *,
+    deprecated_qual_name_usage: str,
+    new_qual_name_usage: str,
+    to_be_removed_in_version: str,
+) -> NoReturn:
+    """Raise :class:`GlotaranDeprectedApiError` error, with formatted message.
+
+    This should only be used if there is no reasonable way to keep the deprecated
+    usage functional!
+
+    Parameters
+    ----------
+    deprecated_qual_name_usage : str
+        Old usage with fully qualified name e.g.:
+        ``'glotaran.read_model_from_yaml(model_yml_str)'``
+    new_qual_name_usage : str
+        New usage as fully qualified name e.g.:
+        ``'glotaran.io.load_model(model_yml_str, format_name="yml_str")'``
+    to_be_removed_in_version : str
+        Version the support for this usage will be removed.
+
+    Raises
+    ------
+    OverDueDeprecation
+        If the current version is greater or equal to ``to_be_removed_in_version``.
+    GlotaranDeprecatedApiError
+        If :class:`OverDueDeprecation` wasn't raised before.
+
+
+    .. # noqa: DAR402 OverDueDeprecation
+    .. # noqa: DAR401 GlotaranDeprecatedApiError
+    """
+    check_overdue(deprecated_qual_name_usage, to_be_removed_in_version)
+    message = (
+        f"Usage of {deprecated_qual_name_usage!r} was deprecated, "
+        f"use {new_qual_name_usage!r} instead.\n"
+        "It wasn't possible to restore the original behavior of this usage "
+        "(most likely due to an object hierarchy change)."
+        "This usage change message won't be show as of version: "
+        f"{to_be_removed_in_version!r}."
+    )
+    raise GlotaranDeprecatedApiError(message)
+
+
 def warn_deprecated(
     *,
     deprecated_qual_name_usage: str,
@@ -166,10 +274,10 @@ def warn_deprecated(
         will import ``package.module.class`` and check if ``class`` has an attribute
         ``mapping``.
 
-    Warns
-    -----
+    Raises
+    ------
     OverDueDeprecation
-        If the current version is greater or equal to ``end_of_life_version``.
+        If the current version is greater or equal to ``to_be_removed_in_version``.
 
     See Also
     --------
@@ -195,16 +303,10 @@ def warn_deprecated(
             )
             return load_model(model_path)
 
+
+    .. # noqa: DAR402
     """
-    if (
-        parse_version(glotaran_version()) >= parse_version(to_be_removed_in_version)
-        and "dev" not in glotaran_version()
-    ):
-        raise OverDueDeprecation(
-            f"Support for {deprecated_qual_name_usage.partition('(')[0]!r} was "
-            f"supposed to be dropped in version: {to_be_removed_in_version!r}\n"
-            f"Current version is: {glotaran_version()!r}"
-        )
+    check_overdue(deprecated_qual_name_usage, to_be_removed_in_version)
     qual_names = (deprecated_qual_name_usage, new_qual_name_usage)
     selected_qual_names = [
         qual_name for qual_name, check in zip(qual_names, check_qual_names) if check
@@ -212,7 +314,7 @@ def warn_deprecated(
     selected_indices = importable_indices[: len(selected_qual_names)]
     check_qualnames_in_tests(qual_names=selected_qual_names, importable_indices=selected_indices)
     warn(
-        DeprecationWarning(
+        GlotaranApiDeprecationWarning(
             f"Usage of {deprecated_qual_name_usage!r} was deprecated, "
             f"use {new_qual_name_usage!r} instead.\n"
             f"This usage will be an error in version: {to_be_removed_in_version!r}."
@@ -265,10 +367,10 @@ def deprecate(
     DecoratedCallable
         Original function or class throwing a Deprecation warning when used.
 
-    Warns
-    -----
+    Raises
+    ------
     OverDueDeprecation
-        If the current version is greater or equal to ``end_of_life_version``.
+        If the current version is greater or equal to ``to_be_removed_in_version``.
 
     See Also
     --------
@@ -321,17 +423,145 @@ def deprecate(
 
     def outer_wrapper(deprecated_object: DecoratedCallable) -> DecoratedCallable:
         """Wrap deprecated_object of all callable kinds."""
-        if type(deprecated_object) is not type:
+        if not isinstance(deprecated_object, type):
             return cast(DecoratedCallable, inject_warn_into_call(deprecated_object))
 
         setattr(
             deprecated_object,
             "__new__",
-            inject_warn_into_call(deprecated_object.__new__),  # type: ignore [arg-type]
+            inject_warn_into_call(deprecated_object.__new__),
         )
-        return deprecated_object
+        return deprecated_object  # type: ignore[return-value]
 
-    return outer_wrapper
+    return cast(Callable[[DecoratedCallable], DecoratedCallable], outer_wrapper)
+
+
+def deprecate_dict_entry(
+    *,
+    dict_to_check: MutableMapping[Hashable, Any],
+    deprecated_usage: str,
+    new_usage: str,
+    to_be_removed_in_version: str,
+    swap_keys: tuple[Hashable, Hashable] | None = None,
+    replace_rules: tuple[Mapping[Hashable, Any], Mapping[Hashable, Any]] | None = None,
+    stacklevel: int = 3,
+) -> None:
+    """Replace dict entry inplace and warn about usage change, if present in the dict.
+
+    Parameters
+    ----------
+    dict_to_check : MutableMapping[Hashable, Any]
+        Dict which should be checked.
+    deprecated_usage : str
+        Old usage to inform user (only used in warning).
+    new_usage : str
+        New usage to inform user (only used in warning).
+    to_be_removed_in_version : str
+        Version the support for this usage will be removed.
+    swap_keys : tuple[Hashable, Hashable]
+        (old_key, new_key),
+        ``dict_to_check[new_key]`` will be assigned the value ``dict_to_check[old_key]``
+        and ``old_key`` will be removed from the dict.
+        by default None
+    replace_rules : Mapping[Hashable, tuple[Any, Any]]
+        ({old_key: old_value}, {new_key: new_value}),
+        If ``dict_to_check[old_key]`` has the value ``old_value``,
+        ``dict_to_check[new_key]`` it will be set to ``new_value``.
+        ``old_key`` will be removed from the dict if ``old_key`` and ``new_key`` aren't equal.
+        by default None
+    stacklevel : int
+        Stack at which the warning should be shown as raise. , by default 3
+
+
+    Raises
+    ------
+    ValueError
+        If both ``swap_keys`` and ``replace_rules`` are None (default) or not None.
+    OverDueDeprecation
+        If the current version is greater or equal to ``to_be_removed_in_version``.
+
+    See Also
+    --------
+    warn_deprecated
+
+    Notes
+    -----
+    To prevent confusion exactly one of ``replace_rules`` and ``swap_keys``
+    needs to be passed.
+
+    Examples
+    --------
+    For readability sake the warnings won't be shown in the examples.
+
+    Swapping key names:
+
+    >>> dict_to_check = {"foo": 123}
+    >>> deprecate_dict_entry(
+            dict_to_check=dict_to_check,
+            deprecated_usage="foo",
+            new_usage="bar",
+            to_be_removed_in_version="0.6.0",
+            swap_keys=("foo", "bar")
+        )
+    >>> dict_to_check
+    {"bar": 123}
+
+    Changing values:
+
+    >>> dict_to_check = {"foo": 123}
+    >>> deprecate_dict_entry(
+            dict_to_check=dict_to_check,
+            deprecated_usage="foo: 123",
+            new_usage="foo: 123.0",
+            to_be_removed_in_version="0.6.0",
+            replace_rules=({"foo": 123}, {"foo": 123.0})
+        )
+    >>> dict_to_check
+    {"foo": 123.0}
+
+    Swapping key names AND changing values:
+
+    >>> dict_to_check = {"type": "kinetic-spectrum"}
+    >>> deprecate_dict_entry(
+            dict_to_check=dict_to_check,
+            deprecated_usage="type: kinectic-spectrum",
+            new_usage="default_megacomplex: decay",
+            to_be_removed_in_version="0.6.0",
+            replace_rules=({"type": "kinetic-spectrum"}, {"default_megacomplex": "decay"})
+        )
+    >>> dict_to_check
+    {"default_megacomplex": "decay"}
+
+
+    .. # noqa: DAR402
+    """
+    dict_changed = False
+
+    if not np.logical_xor(swap_keys is None, replace_rules is None):
+        raise ValueError(
+            "Exactly one of the parameters `swap_keys` or `replace_rules` needs to be provided."
+        )
+    if swap_keys is not None and swap_keys[0] in dict_to_check:
+        dict_changed = True
+        dict_to_check[swap_keys[1]] = dict_to_check[swap_keys[0]]
+        del dict_to_check[swap_keys[0]]
+    if replace_rules is not None:
+        old_key, old_value = next(iter(replace_rules[0].items()))
+        new_key, new_value = next(iter(replace_rules[1].items()))
+        if old_key in dict_to_check and dict_to_check[old_key] == old_value:
+            dict_changed = True
+            dict_to_check[new_key] = new_value
+            if new_key != old_key:
+                del dict_to_check[old_key]
+
+    if dict_changed:
+        warn_deprecated(
+            deprecated_qual_name_usage=deprecated_usage,
+            new_qual_name_usage=new_usage,
+            to_be_removed_in_version=to_be_removed_in_version,
+            stacklevel=stacklevel,
+            check_qual_names=(False, False),
+        )
 
 
 def module_attribute(module_qual_name: str, attribute_name: str) -> Any:
@@ -361,6 +591,7 @@ def deprecate_module_attribute(
     deprecated_qual_name: str,
     new_qual_name: str,
     to_be_removed_in_version: str,
+    module_load_overwrite: str = "",
 ) -> Any:
     """Import and return and anttribute from the new location.
 
@@ -378,12 +609,21 @@ def deprecate_module_attribute(
         ``glotaran.parameter.ParameterGroup``
     to_be_removed_in_version : str
         Version the support for this usage will be removed.
+    module_load_overwrite : str
+        Overwrite the location the functionality will be set from.
+        This allows preserving functionality without polluting a new
+        module with code just for the sake of it. By default ''
 
     Returns
     -------
     Any
         Module attribute from its new location.
 
+    Raises
+    ------
+    OverDueDeprecation
+        If the current version is greater or equal to ``to_be_removed_in_version``.
+
     See Also
     --------
     deprecate
@@ -412,15 +652,23 @@ def deprecate_module_attribute(
 
             raise AttributeError(f"module {__name__} has no attribute {attribute_name}")
 
+
+    .. # noqa: DAR402
     """
-    module_name = ".".join(new_qual_name.split(".")[:-1])
-    attribute_name = new_qual_name.split(".")[-1]
+    if not module_load_overwrite:
+        module_name = ".".join(new_qual_name.split(".")[:-1])
+        attribute_name = new_qual_name.split(".")[-1]
+        check_qual_names = (False, True)
+    else:
+        module_name = ".".join(module_load_overwrite.split(".")[:-1])
+        attribute_name = module_load_overwrite.split(".")[-1]
+        check_qual_names = (False, False)
 
     warn_deprecated(
-        deprecated_qual_name_usage=deprecated_qual_name,
-        new_qual_name_usage=new_qual_name,
+        deprecated_qual_name_usage=re.sub(r"\.__path__$", "", deprecated_qual_name),
+        new_qual_name_usage=re.sub(r"\.__path__$", "", new_qual_name),
         to_be_removed_in_version=to_be_removed_in_version,
-        check_qual_names=(False, True),
+        check_qual_names=check_qual_names,
         stacklevel=4,
         importable_indices=(1, 1),
     )
@@ -432,6 +680,7 @@ def deprecate_submodule(
     deprecated_module_name: str,
     new_module_name: str,
     to_be_removed_in_version: str,
+    module_load_overwrite: str = "",
 ) -> ModuleType:
     r"""Create a module at runtime which retrieves attributes from new module.
 
@@ -451,12 +700,21 @@ def deprecate_submodule(
         ``'glotaran.project.result'``
     to_be_removed_in_version : str
         Version the support for this usage will be removed.
+    module_load_overwrite : str
+        Overwrite the location for the new module the deprecated functionality is loaded from.
+        This allows preserving functionality without polluting a new
+        module with code just for the sake of it. By default ''
 
     Returns
     -------
     ModuleType
         Module containing
 
+    Raises
+    ------
+    OverDueDeprecation
+        If the current version is greater or equal to ``to_be_removed_in_version``.
+
     See Also
     --------
     deprecate
@@ -478,23 +736,32 @@ def deprecate_submodule(
             new_module_name="glotaran.project.result",
             to_be_removed_in_version="0.6.0",
         )
+
+
+    .. # noqa: DAR402
     """
-    new_module = import_module(new_module_name)
+    new_module = (
+        import_module(module_load_overwrite)
+        if module_load_overwrite
+        else import_module(new_module_name)
+    )
+
     deprecated_module = ModuleType(
         deprecated_module_name,
         f"Deprecated use {new_module_name!r} instead.\n\n{new_module.__doc__}",
     )
 
     def warn_getattr(attribute_name: str):
-
         if attribute_name == "__file__":
-            return module_attribute(new_module, attribute_name)
+            return new_module.__file__
 
         elif attribute_name in dir(new_module):
             return deprecate_module_attribute(
                 deprecated_qual_name=f"{deprecated_module_name}.{attribute_name}",
                 new_qual_name=f"{new_module_name}.{attribute_name}",
                 to_be_removed_in_version=to_be_removed_in_version,
+                module_load_overwrite=module_load_overwrite
+                and f"{module_load_overwrite}.{attribute_name}",
             )
 
         raise AttributeError(f"module {deprecated_module_name} has no attribute {attribute_name}")
