@@ -1,15 +1,21 @@
 """Package with deprecation tests and helper functions."""
 from __future__ import annotations
 
+from pathlib import Path
 from typing import TYPE_CHECKING
 
 import pytest
 
+from glotaran.deprecation.deprecation_utils import GlotaranApiDeprecationWarning
+from glotaran.deprecation.deprecation_utils import OverDueDeprecation
+
 if TYPE_CHECKING:
+    from collections.abc import Callable
+    from collections.abc import Mapping
+    from collections.abc import Sequence
     from typing import Any
-    from typing import Callable
-    from typing import Mapping
-    from typing import Sequence
+
+    from _pytest.recwarn import WarningsRecorder
 
 
 def deprecation_warning_on_call_test_helper(
@@ -18,7 +24,7 @@ def deprecation_warning_on_call_test_helper(
     raise_exception=False,
     args: Sequence[Any] = [],
     kwargs: Mapping[str, Any] = {},
-) -> Any:
+) -> tuple[WarningsRecorder, Any]:
     """Helperfunction to quickly test that a deprecated class or function warns.
 
     By default this ignores error when calling the function/class,
@@ -41,17 +47,29 @@ def deprecation_warning_on_call_test_helper(
 
     Returns
     -------
-    Any
-        Return value of deprecated_callable
+    tuple[WarningsRecorder, Any]
+        Tuple of the WarningsRecorder and return value of deprecated_callable
 
     Raises
     ------
     Exception
         Exception caused by deprecated_callable if raise_exception is True.
     """
-    with pytest.warns(DeprecationWarning):
+    with pytest.warns(GlotaranApiDeprecationWarning) as record:
         try:
-            return deprecated_callable(*args, **kwargs)
-        except Exception:
+            result = deprecated_callable(*args, **kwargs)
+
+            assert len(record) >= 1, f"{len(record)=}"
+            assert Path(record[0].filename) == Path(
+                __file__
+            ), f"{Path(record[0].filename)=}, {Path(__file__)=}"
+
+            return record, result
+
+        except OverDueDeprecation as error:
+            raise error
+
+        except Exception as error:
             if raise_exception:
-                raise
+                raise error
+            return record, None
