@@ -12,16 +12,18 @@ See: https://www.python.org/dev/peps/pep-3102/
 
 from __future__ import annotations
 
+from dataclasses import dataclass
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Callable
+    from collections.abc import Callable
+    from typing import Literal
     from typing import Union
 
     import xarray as xr
 
     from glotaran.model import Model
-    from glotaran.parameter import ParameterGroup
+    from glotaran.parameter import Parameters
     from glotaran.project import Result
     from glotaran.project import Scheme
 
@@ -29,6 +31,20 @@ if TYPE_CHECKING:
     DataSaver = Callable[[str, Union[xr.Dataset, xr.DataArray]], None]
 
 
+@dataclass
+class SavingOptions:
+    """A collection of options for result saving."""
+
+    data_filter: list[str] | None = None
+    data_format: Literal["nc"] = "nc"
+    parameter_format: Literal["csv"] = "csv"
+    report: bool = True
+
+
+SAVING_OPTIONS_DEFAULT = SavingOptions()
+SAVING_OPTIONS_MINIMAL = SavingOptions(data_filter=["fitted_data", "residual"], report=False)
+
+
 class DataIoInterface:
     """Baseclass for Data IO plugins."""
 
@@ -37,17 +53,19 @@ class DataIoInterface:
 
         Parameters
         ----------
-        format_name : str
+        format_name: str
             Name of the supported format an instance uses.
         """
         self.format = format_name
 
     def load_dataset(self, file_name: str) -> xr.Dataset | xr.DataArray:
-        """Read data from a file to :xarraydoc:`Dataset` or :xarraydoc:`DataArray` (**NOT IMPLEMENTED**).
+        """Read data from a file to :xarraydoc:`Dataset` or :xarraydoc:`DataArray`.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        file_name : str
+        file_name: str
             File containing the data.
 
         Returns
@@ -66,13 +84,15 @@ class DataIoInterface:
         dataset: xr.Dataset | xr.DataArray,
         file_name: str,
     ):
-        """Save data from :xarraydoc:`Dataset` to a file (**NOT IMPLEMENTED**).
+        """Save data from :xarraydoc:`Dataset` to a file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        dataset : xr.Dataset
+        dataset: xr.Dataset
             Dataset to be saved to file.
-        file_name : str
+        file_name: str
             File to write the data to.
 
 
@@ -90,17 +110,19 @@ class ProjectIoInterface:
 
         Parameters
         ----------
-        format_name : str
+        format_name: str
             Name of the supported format an instance uses.
         """
         self.format = format_name
 
     def load_model(self, file_name: str) -> Model:
-        """Create a Model instance from the specs defined in a file (**NOT IMPLEMENTED**).
+        """Create a Model instance from the specs defined in a file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        file_name : str
+        file_name: str
             File containing the model specs.
 
         Returns
@@ -115,13 +137,15 @@ class ProjectIoInterface:
         raise NotImplementedError(f"Cannot read models with format {self.format!r}")
 
     def save_model(self, model: Model, file_name: str):
-        """Save a Model instance to a spec file (**NOT IMPLEMENTED**).
+        """Save a Model instance to a spec file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
         model: Model
             Model instance to save to specs file.
-        file_name : str
+        file_name: str
             File to write the model specs to.
 
 
@@ -130,18 +154,20 @@ class ProjectIoInterface:
         """
         raise NotImplementedError(f"Cannot save models with format {self.format!r}")
 
-    def load_parameters(self, file_name: str) -> ParameterGroup:
-        """Create a ParameterGroup instance from the specs defined in a file (**NOT IMPLEMENTED**).
+    def load_parameters(self, file_name: str) -> Parameters:
+        """Create a Parameters instance from the specs defined in a file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        file_name : str
+        file_name: str
             File containing the parameter specs.
 
         Returns
         -------
-        ParameterGroup
-            ParameterGroup instance created from the file.
+        Parameters
+            Parameters instance created from the file.
 
 
         .. # noqa: DAR202
@@ -149,14 +175,16 @@ class ProjectIoInterface:
         """
         raise NotImplementedError(f"Cannot read parameters with format {self.format!r}")
 
-    def save_parameters(self, parameters: ParameterGroup, file_name: str):
-        """Save a ParameterGroup instance to a spec file (**NOT IMPLEMENTED**).
+    def save_parameters(self, parameters: Parameters, file_name: str):
+        """Save a Parameters instance to a spec file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        parameters : ParameterGroup
-            ParameterGroup instance to save to specs file.
-        file_name : str
+        parameters: Parameters
+            Parameters instance to save to specs file.
+        file_name: str
             File to write the parameter specs to.
 
 
@@ -166,11 +194,13 @@ class ProjectIoInterface:
         raise NotImplementedError(f"Cannot save parameters with format {self.format!r}")
 
     def load_scheme(self, file_name: str) -> Scheme:
-        """Create a Scheme instance from the specs defined in a file (**NOT IMPLEMENTED**).
+        """Create a Scheme instance from the specs defined in a file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        file_name : str
+        file_name: str
             File containing the parameter specs.
 
         Returns
@@ -184,13 +214,15 @@ class ProjectIoInterface:
         raise NotImplementedError(f"Cannot read scheme with format {self.format!r}")
 
     def save_scheme(self, scheme: Scheme, file_name: str):
-        """Save a Scheme instance to a spec file (**NOT IMPLEMENTED**).
+        """Save a Scheme instance to a spec file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        scheme : Scheme
+        scheme: Scheme
             Scheme instance to save to specs file.
-        file_name : str
+        file_name: str
             File to write the scheme specs to.
 
 
@@ -200,11 +232,13 @@ class ProjectIoInterface:
         raise NotImplementedError(f"Cannot save scheme with format {self.format!r}")
 
     def load_result(self, result_path: str) -> Result:
-        """Create a Result instance from the specs defined in a file (**NOT IMPLEMENTED**).
+        """Create a Result instance from the specs defined in a file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        result_path : str
+        result_path: str
             Path containing the result data.
 
         Returns
@@ -218,15 +252,25 @@ class ProjectIoInterface:
         """
         raise NotImplementedError(f"Cannot read result with format {self.format!r}")
 
-    def save_result(self, result: Result, result_path: str):
-        """Save a Result instance to a spec file (**NOT IMPLEMENTED**).
+    def save_result(
+        self,
+        result: Result,
+        result_path: str,
+        *,
+        saving_options: SavingOptions = SAVING_OPTIONS_DEFAULT,
+    ) -> list[str]:
+        """Save a Result instance to a spec file.
+
+        **NOT IMPLEMENTED**
 
         Parameters
         ----------
-        result : Result
+        result: Result
             Result instance to save to specs file.
-        result_path : str
+        result_path: str
             Path to write the result data to.
+        saving_options: SavingOptions
+            Options for the saved result.
 
 
         .. # noqa: DAR101
