@@ -8,8 +8,10 @@ and causing an [override] type error in the plugins implementation.
 """
 from __future__ import annotations
 
+from pathlib import Path
 from typing import TYPE_CHECKING
 
+import xarray as xr
 from tabulate import tabulate
 
 from glotaran.io.interface import DataIoInterface
@@ -22,22 +24,23 @@ from glotaran.plugin_system.base_registry import methods_differ_from_baseclass_t
 from glotaran.plugin_system.base_registry import registered_plugins
 from glotaran.plugin_system.base_registry import set_plugin
 from glotaran.plugin_system.base_registry import show_method_help
+from glotaran.plugin_system.base_registry import supported_file_extensions
 from glotaran.plugin_system.io_plugin_utils import bool_table_repr
-from glotaran.plugin_system.io_plugin_utils import inferr_file_format
+from glotaran.plugin_system.io_plugin_utils import infer_file_format
 from glotaran.plugin_system.io_plugin_utils import not_implemented_to_value_error
 from glotaran.plugin_system.io_plugin_utils import protect_from_overwrite
 from glotaran.utils.ipython import MarkdownStr
 
 if TYPE_CHECKING:
-    from os import PathLike
+    from collections.abc import Callable
+    from collections.abc import Generator
+    from collections.abc import Sequence
     from typing import Any
-    from typing import Callable
     from typing import Literal
 
-    import xarray as xr
-
     from glotaran.io.interface import DataLoader
     from glotaran.io.interface import DataSaver
+    from glotaran.typing import StrOrPath
 
 DATA_IO_METHODS = ("load_dataset", "save_dataset")
 """Methods used by DataIoInterface plugins."""
@@ -171,13 +174,13 @@ def get_data_io(format_name: str) -> DataIoInterface:
 
 @not_implemented_to_value_error
 def load_dataset(
-    file_name: str | PathLike[str], format_name: str = None, **kwargs: Any
-) -> xr.Dataset | xr.DataArray:
+    file_name: StrOrPath, format_name: str | None = None, **kwargs: Any
+) -> xr.Dataset:
     """Read data from a file to :xarraydoc:`Dataset` or :xarraydoc:`DataArray`.
 
     Parameters
     ----------
-    file_name : str | PathLike[str]
+    file_name : StrOrPath
         File containing the data.
     format_name : str
         Format the file is in, if not provided it will be inferred from the file extension.
@@ -188,20 +191,28 @@ def load_dataset(
 
     Returns
     -------
-    xr.Dataset|xr.DataArray
+    xr.Dataset
         Data loaded from the file.
     """
-    io = get_data_io(format_name or inferr_file_format(file_name))
-    return io.load_dataset(str(file_name), **kwargs)  # type: ignore[call-arg]
+    io = get_data_io(format_name or infer_file_format(file_name))
+    dataset = io.load_dataset(Path(file_name).as_posix(), **kwargs)
+
+    if isinstance(dataset, xr.DataArray):
+        dataset = dataset.to_dataset(name="data")
+    dataset.attrs["loader"] = load_dataset
+    dataset.attrs["source_path"] = Path(file_name).as_posix()
+    return dataset
 
 
 @not_implemented_to_value_error
 def save_dataset(
     dataset: xr.Dataset | xr.DataArray,
-    file_name: str | PathLike[str],
-    format_name: str = None,
+    file_name: StrOrPath,
+    format_name: str | None = None,
     *,
+    data_filters: list[str] | None = None,
     allow_overwrite: bool = False,
+    update_source_path: bool = True,
     **kwargs: Any,
 ) -> None:
     """Save data from :xarraydoc:`Dataset` or :xarraydoc:`DataArray` to a file.
@@ -210,24 +221,30 @@ def save_dataset(
     ----------
     dataset : xr.Dataset | xr.DataArray
         Data to be written to file.
-    file_name : str | PathLike[str]
+    file_name : StrOrPath
         File to write the data to.
     format_name : str
         Format the file should be in, if not provided it will be inferred from the file extension.
+    data_filters : list[str] | None
+        Optional list of items in the dataset to be saved.
     allow_overwrite : bool
         Whether or not to allow overwriting existing files, by default False
+    update_source_path: bool
+        Whether or not to update the ``source_path`` attribute to ``file_name`` when saving.
+        by default True
     **kwargs : Any
         Additional keyword arguments passes to the ``write_dataset`` implementation
-        of the data io plugin. If you aren't sure about those use ``get_datawriter``
+        of the data io plugin. If you aren't sure about those use ``get_datasaver``
         to get the implementation with the proper help and autocomplete.
     """
     protect_from_overwrite(file_name, allow_overwrite=allow_overwrite)
-    io = get_data_io(format_name or inferr_file_format(file_name, needs_to_exist=False))
-    io.save_dataset(  # type: ignore[call-arg]
-        file_name=str(file_name),
-        dataset=dataset,
-        **kwargs,
-    )
+    io = get_data_io(format_name or infer_file_format(file_name, needs_to_exist=False))
+    if "loader" in dataset.attrs:
+        del dataset.attrs["loader"]
+    io.save_dataset(file_name=Path(file_name).as_posix(), dataset=dataset, **kwargs)
+    dataset.attrs["loader"] = load_dataset
+    if update_source_path is True or "source_path" not in dataset.attrs:
+        dataset.attrs["source_path"] = Path(file_name).as_posix()
 
 
 def get_dataloader(format_name: str) -> DataLoader:
@@ -302,7 +319,7 @@ def data_io_plugin_table(*, plugin_names: bool = False, full_names: bool = False
 
     Returns
     -------
-    str
+    MarkdownStr
         Markdown table of data io plugins.
     """
     table_data = methods_differ_from_baseclass_table(
@@ -315,9 +332,37 @@ def data_io_plugin_table(*, plugin_names: bool = False, full_names: bool = False
     header_values = ["Format name", *DATA_IO_METHODS]
     if plugin_names:
         header_values.append("Plugin name")
-    headers = tuple(map(lambda x: f"__{x}__", header_values))
+    headers = tuple(f"__{x}__" for x in header_values)
     return MarkdownStr(
         tabulate(
             bool_table_repr(table_data), tablefmt="github", headers=headers, stralign="center"
         )
     )
+
+
+def supported_file_extensions_data_io(
+    method_names: str | Sequence[str],
+) -> Generator[str, None, None]:
+    """Get data io formats that support all methods in ``method_names``.
+
+    Parameters
+    ----------
+    method_names: str | Sequence[str]
+        Names of Methods that need to support the file extension.
+
+    Yields
+    ------
+    Generator[str, None, None]
+        File extension supported by all methods in ``method_names``.
+
+    See Also
+    --------
+    supported_file_extensions
+    DATA_IO_METHODS
+    """
+    yield from supported_file_extensions(
+        method_names,
+        known_data_formats(),
+        get_data_io,
+        DataIoInterface,
+    )
