@@ -1,9 +1,8 @@
 from __future__ import annotations
 
+from collections.abc import MutableMapping
 from copy import copy
 from typing import TYPE_CHECKING
-from typing import MutableMapping
-from typing import Type
 from typing import cast
 from warnings import warn
 
@@ -11,10 +10,10 @@ import pytest
 
 from glotaran.builtin.io.sdt.sdt_file_reader import SdtDataIo
 from glotaran.builtin.io.yml.yml import YmlProjectIo
-from glotaran.builtin.models.kinetic_image import KineticImageModel
+from glotaran.builtin.megacomplexes.decay import DecayMegacomplex
 from glotaran.io.interface import DataIoInterface
 from glotaran.io.interface import ProjectIoInterface
-from glotaran.model.base_model import Model
+from glotaran.model.megacomplex import Megacomplex
 from glotaran.plugin_system.base_registry import PluginOverwriteWarning
 from glotaran.plugin_system.base_registry import add_instantiated_plugin_to_registry
 from glotaran.plugin_system.base_registry import add_plugin_to_registry
@@ -27,6 +26,7 @@ from glotaran.plugin_system.base_registry import methods_differ_from_baseclass_t
 from glotaran.plugin_system.base_registry import registered_plugins
 from glotaran.plugin_system.base_registry import set_plugin
 from glotaran.plugin_system.base_registry import show_method_help
+from glotaran.plugin_system.base_registry import supported_file_extensions
 
 if TYPE_CHECKING:
     from _pytest.capture import CaptureFixture
@@ -46,18 +46,43 @@ class MockPlugin:
         pass
 
 
-class MockPluginSubclass(MockPlugin):
+class MockPluginSubclassPartial(MockPlugin):
     def some_method(self):
         return "different implementation"
 
 
+class MockPluginSubclassFull(MockPlugin):
+    def some_method(self):
+        return "different implementation"
+
+    def some_other_method(self):
+        return "different implementation"
+
+
+class MockPluginSubclassStr(MockPlugin):
+    def some_method(self):
+        return "different implementation"
+
+
+def get_mock_plugin_function(plugin_registry_key: str):
+    if plugin_registry_key == "base":
+        return MockPlugin
+    elif plugin_registry_key == "sub_class_partial":
+        return MockPluginSubclassPartial
+    elif plugin_registry_key == "sub_class_full":
+        return MockPluginSubclassFull
+    elif plugin_registry_key == "sub_class_str":
+        return MockPluginSubclassStr
+    raise ValueError(f"No mock plugin with name {plugin_registry_key!r}")
+
+
 mock_registry_data_io = cast(
     MutableMapping[str, DataIoInterface],
     {
         "sdt": SdtDataIo("sdt"),
         "yml": YmlProjectIo("yml"),
         "glotaran.builtin.io.sdt.sdt_file_reader.SdtDataIo": SdtDataIo("sdt"),
-        "kinetic-image": KineticImageModel,
+        "decay": DecayMegacomplex,
         "mock_plugin": MockPlugin,
         "imported_plugin": MockPlugin,
     },
@@ -66,7 +91,7 @@ mock_registry_data_io = cast(
 mock_registry_project_io = cast(
     MutableMapping[str, ProjectIoInterface], copy(mock_registry_data_io)
 )
-mock_registry_model = cast(MutableMapping[str, Type[Model]], copy(mock_registry_data_io))
+mock_registry_model = cast(MutableMapping[str, type[Megacomplex]], copy(mock_registry_data_io))
 
 
 @pytest.mark.parametrize(
@@ -103,7 +128,7 @@ def test_PluginOverwriteWarning():
             "to use 'glotaran.builtin.io.sdt.sdt_file_reader.SdtDataIo' instead."
         )
 
-        assert record[0].message.args[0] == expected  # type: ignore
+        assert record[0].message.args[0] == expected
 
 
 @pytest.mark.parametrize(
@@ -111,7 +136,7 @@ def test_PluginOverwriteWarning():
     (
         ("sdt_new", SdtDataIo("sdt"), copy(mock_registry_data_io)),
         ("yml_new", YmlProjectIo("yml"), copy(mock_registry_project_io)),
-        ("kinetic-image_new", KineticImageModel, copy(mock_registry_model)),
+        ("decay_new", DecayMegacomplex, copy(mock_registry_model)),
     ),
 )
 def test_add_plugin_to_register(
@@ -134,9 +159,7 @@ def test_add_plugin_to_register_naming_error():
             r"you provided the name 'bad\.name'."
         ),
     ):
-        add_plugin_to_registry(
-            "bad.name", MockPlugin, mock_registry_model, "set_plugin"  # type:ignore
-        )
+        add_plugin_to_registry("bad.name", MockPlugin, mock_registry_model, "set_plugin")
 
 
 def test_add_plugin_to_register_existing_plugin():
@@ -144,12 +167,7 @@ def test_add_plugin_to_register_existing_plugin():
 
     plugin_registry = copy(mock_registry_data_io)
     with pytest.warns(PluginOverwriteWarning, match="SdtDataIo.+sdt.+YmlProjectIo") as record:
-        add_plugin_to_registry(
-            "sdt",
-            YmlProjectIo("sdt"),
-            plugin_registry,  # type:ignore
-            "set_plugin",
-        )
+        add_plugin_to_registry("sdt", YmlProjectIo("sdt"), plugin_registry, "set_plugin")
         assert len(record) == 1
     assert plugin_registry["glotaran.builtin.io.yml.yml.YmlProjectIo"].format == "sdt"
 
@@ -253,7 +271,7 @@ def test_registered_plugins():
     result = [
         "sdt",
         "yml",
-        "kinetic-image",
+        "decay",
         "mock_plugin",
         "imported_plugin",
     ]
@@ -326,10 +344,12 @@ def test_show_method_help(capsys: CaptureFixture, plugin: MockPlugin | type[Mock
 @pytest.mark.parametrize(
     "method_names,plugin,expected",
     (
-        ("some_method", MockPluginSubclass, [True]),
-        ("some_method", MockPluginSubclass(), [True]),
-        (["some_method", "some_other_method"], MockPluginSubclass, [True, False]),
-        (["some_method", "some_other_method"], MockPluginSubclass(), [True, False]),
+        ("some_method", MockPluginSubclassPartial, [True]),
+        ("some_method", MockPluginSubclassPartial(), [True]),
+        (["some_method", "some_other_method"], MockPluginSubclassPartial, [True, False]),
+        (["some_method", "some_other_method"], MockPluginSubclassPartial(), [True, False]),
+        (["some_method", "some_other_method"], MockPluginSubclassFull, [True, True]),
+        (["some_method", "some_other_method"], MockPluginSubclassFull(), [True, True]),
     ),
 )
 def test_methods_differ_from_baseclass(
@@ -338,22 +358,23 @@ def test_methods_differ_from_baseclass(
     """Inherited methods are the same as base class and overwritten ones differ"""
     result = methods_differ_from_baseclass(method_names, plugin, MockPlugin)
 
-    assert result == expected
+    assert list(result) == expected
 
 
 @pytest.mark.parametrize(
     "method_names,plugin_registry_keys,expected",
     (
         ("some_method", "base", [["`base`", False]]),
-        ("some_method", "sub_class", [["`sub_class`", True]]),
-        ("some_method", "sub_class_inst", [["`sub_class_inst`", True]]),
+        ("some_method", "sub_class_partial", [["`sub_class_partial`", True]]),
+        ("some_method", "sub_class_full", [["`sub_class_full`", True]]),
         (
             ["some_method", "some_other_method"],
-            ["base", "sub_class", "sub_class_inst"],
+            ["base", "sub_class_partial", "sub_class_full", "sub_class_str"],
             [
                 ["`base`", False, False],
-                ["`sub_class`", True, False],
-                ["`sub_class_inst`", True, False],
+                ["`sub_class_partial`", True, False],
+                ["`sub_class_full`", True, True],
+                ["`sub_class_str`", True, False],
             ],
         ),
     ),
@@ -365,30 +386,41 @@ def test_methods_differ_from_baseclass_table(
 ):
     """Inherited methods are the same as base class and overwritten ones differ"""
 
-    def get_plugin_function(plugin_registry_key: str):
-        if plugin_registry_key == "base":
-            return MockPlugin
-        elif plugin_registry_key in ["sub_class", "sub_class_inst"]:
-            return MockPluginSubclass
-
     result = methods_differ_from_baseclass_table(
-        method_names, plugin_registry_keys, get_plugin_function, MockPlugin
+        method_names, plugin_registry_keys, get_mock_plugin_function, MockPlugin
     )
 
-    assert result == expected
+    assert list(result) == expected
 
 
 def test_methods_differ_from_baseclass_table_plugin_names():
     """Show plugin name"""
 
-    def get_plugin_function(plugin_registry_key: str):
-        if plugin_registry_key == "base":
-            return MockPlugin
-        elif plugin_registry_key == "sub_class":
-            return MockPluginSubclass
-
     result = methods_differ_from_baseclass_table(
-        "some_method", "base", get_plugin_function, MockPlugin, plugin_names=True
+        "some_method", "base", get_mock_plugin_function, MockPlugin, plugin_names=True
     )
 
-    assert result == [["`base`", False, "`test_base_registry.MockPlugin`"]]
+    assert list(result) == [["`base`", False, "`test_base_registry.MockPlugin`"]]
+
+
+@pytest.mark.parametrize(
+    "method_names, expected",
+    (
+        ("some_method", [".sub_class_partial", ".sub_class_full"]),
+        ("some_other_method", [".sub_class_full"]),
+        (["some_method", "some_other_method"], [".sub_class_full"]),
+    ),
+)
+def test_supported_file_extensions(
+    method_names: str | list[str],
+    expected: list[str],
+):
+    """Only extensions where the plugin supports all methods in ``method_names`` are returned."""
+    result = supported_file_extensions(
+        method_names,
+        ["base", "sub_class_partial", "sub_class_full", "sub_class_str"],
+        get_mock_plugin_function,
+        MockPlugin,
+    )
+
+    assert list(result) == expected
