@@ -2,15 +2,16 @@
 from __future__ import annotations
 
 from collections import UserString
-from os import PathLike
+
+from glotaran.plugin_system.io_plugin_utils import infer_file_format
+from glotaran.typing.types import StrOrPath
 
 
 class MarkdownStr(UserString):
     """String wrapper class for rich display integration of markdown in ipython."""
 
-    def __init__(self, wrapped_str: str, *, syntax: str = None):
-        """String class automatically displayed as markdown by ipython.
-
+    def __init__(self, wrapped_str: str, *, syntax: str | None = None):
+        """Initialize string class that is automatically displayed as markdown by ``ipython``.
 
         Parameters
         ----------
@@ -23,13 +24,16 @@ class MarkdownStr(UserString):
         ----
         Possible syntax highlighting values can e.g. be found here:
         https://support.codebasehq.com/articles/tips-tricks/syntax-highlighting-in-markdown
+
+
+        .. # noqa: DAR101
         """
         # This needs to be called data since ipython is looking for this attr
         self.data = str(wrapped_str)
         self.syntax = syntax
 
     def _repr_markdown_(self) -> str:
-        """Special method used by ``ipython`` to render markdown.
+        """Render markdown automatically when in a ``ipython`` context.
 
         See:
         https://ipython.readthedocs.io/en/latest/config/integrating.html?highlight=_repr_markdown_#rich-display
@@ -57,20 +61,24 @@ class MarkdownStr(UserString):
             return NotImplemented
 
 
-def display_file(path: str | PathLike[str], *, syntax: str = None) -> MarkdownStr:
+def display_file(path: StrOrPath, *, syntax: str | None = None) -> MarkdownStr:
     """Display a file with syntax highlighting ``syntax``.
 
     Parameters
     ----------
-    path : str | PathLike[str]
+    path : StrOrPath
         Paths to the file
-    syntax : str, optional
-        Syntax highlighting which should be applied, by default None
+    syntax: str | None
+        Syntax used for syntax highlighting. Defaults to None which means that the syntax is
+        inferred based on the file extension. Pass the value ``""`` to deactivate syntax
+        highlighting.
 
     Returns
     -------
     MarkdownStr
         File content with syntax highlighting to render in ipython.
     """
+    if syntax is None:
+        syntax = infer_file_format(path)
     with open(path, encoding="utf8") as file:
         return MarkdownStr(file.read(), syntax=syntax)
